@@ -12,7 +12,12 @@
 #include <FL/fl_draw.H>
 #include <FL/Fl_Menu_Bar.H>
 #include <FL/Fl_Menu_Item.H>
+#ifdef CALCULATOR
 #include "fl_adjustable_file_chooser.H"		// FLTK file chooser
+#endif
+#ifdef WORKSHEET
+#include <Fl/Fl_File_Chooser.H>
+#endif
 #include <stdlib.h>
 #include "yacas.h"
 #include "editor.h"
@@ -25,6 +30,8 @@
 #include "standard.h"
 #include "errors.h"
 #define InternalEval aEnvironment.iEvaluator->Eval
+#define RESULT aEnvironment.iStack.GetElement(aStackTop)
+#define ARGUMENT(i) aEnvironment.iStack.GetElement(aStackTop+i)
 
 
 
@@ -245,7 +252,12 @@ void cb_menu_insert(Fl_Widget* o, void* v)
 void cb_notepad(Fl_Widget* o, void* v)
 {
     char *newfile;
+#ifdef CALCULATOR
     newfile = fl_adjustable_file_chooser("Open File?", "*", "");
+#endif
+#ifdef WORKSHEET
+    newfile = fl_file_chooser("Open file?","*","");
+#endif
     if (newfile != NULL)
     {
         console->LoadNotePad(newfile);
@@ -257,61 +269,48 @@ void cb_notepad(Fl_Widget* o, void* v)
 #include "proteusmenu.h"
 
 
-void LispExit(LispEnvironment& aEnvironment, LispPtr& aResult,
-              LispPtr& aArguments)
+void LispExit(LispEnvironment& aEnvironment,LispInt aStackTop)
 {
     exit(0);
 //    InternalTrue(aEnvironment, aResult);
 }
 
-void LispShowInput(LispEnvironment& aEnvironment, LispPtr& aResult,
-              LispPtr& aArguments)
+void LispShowInput(LispEnvironment& aEnvironment,LispInt aStackTop)
 {
-    TESTARGS(2);
-    LispPtr evaluated;
-    InternalEval(aEnvironment, evaluated, Argument(aArguments,1));
-    if (IsTrue(aEnvironment, evaluated))
+    if (IsTrue(aEnvironment, ARGUMENT(1)))
     {
         console->ShowInput(1);
     }
-    if (IsFalse(aEnvironment, evaluated))
+    if (IsFalse(aEnvironment, ARGUMENT(1)))
     {
         console->ShowInput(0);
     }
-    InternalTrue(aEnvironment,aResult);
+    InternalTrue(aEnvironment,RESULT);
 }
 
-void LispEnableInput(LispEnvironment& aEnvironment, LispPtr& aResult,
-              LispPtr& aArguments)
+void LispEnableInput(LispEnvironment& aEnvironment,LispInt aStackTop)
 {
-    TESTARGS(2);
     LispPtr evaluated;
-    InternalEval(aEnvironment, evaluated, Argument(aArguments,1));
-    if (IsTrue(aEnvironment, evaluated))
+    if (IsTrue(aEnvironment, ARGUMENT(1)))
     {
         console->EnableInput(1);
     }
-    if (IsFalse(aEnvironment, evaluated))
+    if (IsFalse(aEnvironment, ARGUMENT(1)))
     {
         console->EnableInput(0);
     }
-    InternalTrue(aEnvironment,aResult);
+    InternalTrue(aEnvironment,RESULT);
 }
 
-void LispNotepad(LispEnvironment& aEnvironment, LispPtr& aResult,
-              LispPtr& aArguments)
+void LispNotepad(LispEnvironment& aEnvironment,LispInt aStackTop)
 {
-    TESTARGS(2);
-    LispPtr evaluated;
-    InternalEval(aEnvironment, evaluated, Argument(aArguments,1));
-
-    CHK_ARG(evaluated.Get() != NULL, 1);
-    LispStringPtr orig = evaluated.Get()->String();
-    CHK_ARG(orig != NULL, 1);
+    CHK_ARG_CORE(ARGUMENT(1).Get() != NULL, 1);
+    LispStringPtr orig = ARGUMENT(1).Get()->String();
+    CHK_ARG_CORE(orig != NULL, 1);
     LispString oper;
     InternalUnstringify(oper, orig);
     console->LoadNotePad(&oper[0]);
-    InternalTrue(aEnvironment,aResult);
+    InternalTrue(aEnvironment,RESULT);
 }
 
 
@@ -322,7 +321,9 @@ void GetProteusConfiguration()
     strcpy(defdir,".\\");
     return;
 #else
-    FILE*f = fopen("/etc/proteus.conf","r");
+    FILE*f;
+    f = fopen("/etc/proteus.conf","r");
+    if (!f) f=fopen("proteus.conf","r");
     if (f)
     {
         fscanf(f,"%s",defdir);
@@ -344,15 +345,25 @@ void RestartYacas()
     line=0;
 
     yacas = CYacas::NewL(new StringOutput(the_out));
+
+#define CORE_KERNEL_FUNCTION(iname,fname,nrargs,flags) (*yacas)()().SetCommand(fname,iname,nrargs,flags)
+CORE_KERNEL_FUNCTION("NoteShowInput",LispShowInput,1,YacasEvaluator::Function | YacasEvaluator::Fixed);
+CORE_KERNEL_FUNCTION("NoteEnableInput",LispEnableInput,1,YacasEvaluator::Function | YacasEvaluator::Fixed);
+CORE_KERNEL_FUNCTION("Notepad",LispNotepad,1,YacasEvaluator::Function | YacasEvaluator::Fixed);
+CORE_KERNEL_FUNCTION("Exit",LispExit,0,YacasEvaluator::Function | YacasEvaluator::Fixed);
+#undef CORE_KERNEL_FUNCTION
+
+
+/*TODO remove old
     (*yacas)()().Commands().SetAssociation(LispEvaluator(LispExit),
                                            (*yacas)()().HashTable().LookUp("Exit"));
-
     (*yacas)()().Commands().SetAssociation(LispEvaluator(LispShowInput),
                                            (*yacas)()().HashTable().LookUp("NoteShowInput"));
     (*yacas)()().Commands().SetAssociation(LispEvaluator(LispEnableInput),
                                            (*yacas)()().HashTable().LookUp("NoteEnableInput"));
     (*yacas)()().Commands().SetAssociation(LispEvaluator(LispNotepad),
                                            (*yacas)()().HashTable().LookUp("Notepad"));
+*/
 
 
     char cmd[128];
@@ -406,111 +417,98 @@ void quit_cb(Fl_Widget *,void *)
 
 int main(int argc, char **argv)
 {
-    GetProteusConfiguration();
-    curoutlen=10;
-    outbuf = (char*)malloc(curoutlen);
+  GetProteusConfiguration();
+  curoutlen=10;
+  outbuf = (char*)malloc(curoutlen);
 
-    Fl_Window* w;
+  Fl_Window* w;
+  {
+    Fl_Window* o = /* foo_window = */ new Fl_Window(640, 320);
+    w = o;
     {
-        Fl_Window* o = /* foo_window = */ new Fl_Window(640, 320);
-
-        w = o;
+      mainTabs = new Fl_Tabs(5, 1, 630, 315);
+      o->selection_color(15);
+      {
+        Fl_Group* o = input = new Fl_Group(10, 20, 630, 310, "Input");
         {
-            mainTabs = new Fl_Tabs(5, 1, 630, 315);
-            o->selection_color(15);
-
-            {
-                Fl_Group* o = input = new Fl_Group(10, 20, 630, 310, "Input");
-				{
-				menubar = new Fl_Menu_Bar(11,23,618,20);
-				menubar->menu(menuitems);
-				menubar->textfont(10);
-				menubar->textsize(12);
-				}
-				{
-                console = new FltkConsole(11,43,618,265,12);
-				}
-                o->end();
-                Fl_Group::current()->resizable(o);
-            }
-
-            {
-                Fl_Group* o = grapher = new Fl_Group(10, 20, 630, 310, "Graph");
-                drawing = new Drawer(11,23,618,285);
-                o->end();
-            }
-
-            {
-                Fl_Group* o = new Fl_Group(10, 20, 630, 310, "Edit");
-                editor_add_items(11,23,618, 285);
-                o->end();
-            }
-
-            {
-                Fl_Group* helptab;
-                Fl_Group* o = helptab = new Fl_Group(10, 20, 630, 310, "Help");
-
-
-                {
-                    HelpView* o = helpview_ = new HelpView(11, 23, 620, 260);
-                    o->box(FL_DOWN_BOX);
-                    o->callback((Fl_Callback*)cb_helpview_);
-                    o->end();
-                    Fl_Group::current()->resizable(o);
-                }
-                {
-                    Fl_Button* o = back_ = new Fl_Button(430, 285, 25, 25, "@<");
-                    o->shortcut(0xff51);
-                    o->labeltype(FL_SYMBOL_LABEL);
-                    o->callback((Fl_Callback*)cb_back_);
-                }
-                {
-                    Fl_Button* o = forward_ = new Fl_Button(465, 285, 25, 25, "@>");
-                    o->shortcut(0xff53);
-                    o->labeltype(FL_SYMBOL_LABEL);
-                    o->callback((Fl_Callback*)cb_forward_);
-                }
-               {
-                    Fl_Button* o = smaller_ = new Fl_Button(360, 285, 25, 25, "F");
-                    o->labelfont(1);
-                    o->labelsize(9);
-                    o->callback((Fl_Callback*)cb_smaller_);
-                }
-                {
-                    Fl_Button* o = larger_ = new Fl_Button(395, 285, 25, 25, "F");
-                    o->labelfont(11);
-                    o->labelsize();
-                    o->callback((Fl_Callback*)cb_larger_);
-                }
-                {
-                    char helpfile[128];
-#ifdef WIN32
-                    sprintf(helpfile,"proteusbooks.html");
-#else
-                    sprintf(helpfile,"%sdocumentation/%s",defdir,YACAS_DOC);
-#endif
-                    helpview_->load(helpfile);
-                }
-
-                o->end();
-
-            }
-
-            init_editor();
-
-            mainTabs->end();
-            Fl_Group::current()->resizable(o);
-
+          menubar = new Fl_Menu_Bar(11,23,618,20);
+          menubar->menu(menuitems);
+          menubar->textfont(10);
+          menubar->textsize(12);
         }
-        Fl_Group::current()->resizable(o);
+        console = new FltkConsole(11,43,618,265,12);
         o->end();
-    }
+        Fl_Group::current()->resizable(o);
+      }
+      {
+        Fl_Group* o = grapher = new Fl_Group(10, 20, 630, 310, "Graph");
+        drawing = new Drawer(11,23,618,285);
+        o->end();
+      }
+      {
+        Fl_Group* o = new Fl_Group(10, 20, 630, 310, "Edit");
+         editor_add_items(11,23,618, 285);
+        o->end();
+      }
+      {
+        Fl_Group* helptab;
+        Fl_Group* o = helptab = new Fl_Group(10, 20, 630, 310, "Help");
+        {
+          HelpView* o = helpview_ = new HelpView(11, 23, 620, 260);
+          o->box(FL_DOWN_BOX);
+          o->callback((Fl_Callback*)cb_helpview_);
+          o->end();
+          Fl_Group::current()->resizable(o);
+        }
+        {
+          Fl_Button* o = back_ = new Fl_Button(430, 285, 25, 25, "@<");
+          o->shortcut(0xff51);
+          o->labeltype(FL_SYMBOL_LABEL);
+          o->callback((Fl_Callback*)cb_back_);
+        }
+        {
+          Fl_Button* o = forward_ = new Fl_Button(465, 285, 25, 25, "@>");
+          o->shortcut(0xff53);
+          o->labeltype(FL_SYMBOL_LABEL);
+          o->callback((Fl_Callback*)cb_forward_);
+        }
+        {
+          Fl_Button* o = smaller_ = new Fl_Button(360, 285, 25, 25, "F");
+          o->labelfont(1);
+          o->labelsize(9);
+          o->callback((Fl_Callback*)cb_smaller_);
+        }
+        {
+          Fl_Button* o = larger_ = new Fl_Button(395, 285, 25, 25, "F");
+          o->labelfont(11);
+          o->labelsize();
+          o->callback((Fl_Callback*)cb_larger_);
+        }
+        {
+          char helpfile[128];
+#ifdef WIN32
+          sprintf(helpfile,"proteusbooks.html");
+#else
+          sprintf(helpfile,"%sdocumentation/%s",defdir,YACAS_DOC);
+#endif
+          helpview_->load(helpfile);
+        }
+        o->end();
+      }
+      init_editor();
+      mainTabs->end();
+      Fl_Group::current()->resizable(o);
 
-    RestartYacas();
-    w->callback(quit_cb);
-    w->show(argc, argv);
-    atexit(myexit);
-    return Fl::run();
+    }
+    Fl_Group::current()->resizable(o);
+    o->end();
+  }
+
+  RestartYacas();
+  w->callback(quit_cb);
+  w->show(argc, argv);
+  atexit(myexit);
+  return Fl::run();
 }
 
 #endif
@@ -581,259 +579,257 @@ int main(int argc, char **argv)
 
     Fl_Window* w;
     {
-        Fl_Window* o = /* foo_window = */ new Fl_Window(160, 220);
+      Fl_Window* o = /* foo_window = */ new Fl_Window(160, 220);
 
-        w = o;
+      w = o;
+      {
+        mainTabs = new Fl_Tabs(0, 1, 160, 219);
+        o->selection_color(15);
         {
-            mainTabs = new Fl_Tabs(0, 1, 160, 219);
-            o->selection_color(15);
-
+            Fl_Group* o = input = new Fl_Group(2, 15, 158, 215, "Input");
             {
-                Fl_Group* o = input = new Fl_Group(2, 15, 158, 215, "Input");
-
-				{
-                	console = new FltkConsole(2,16,156,153,9);
-				}
-				{
-					menubar = new Fl_Menu_Bar(2, 169, 156, 15);
-					menubar->menu(menuitems);
-					menubar->textfont(8);
-					menubar->textsize(10);
-				}
-				{
-					Fl_Button* o = num_seven_ = new Fl_Button(2,184,12,12,"7");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"7");
-				}
-				{
-					Fl_Button* o = num_four_ = new Fl_Button(2,196,12,12,"4");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"4");
-				}
-				{
-					Fl_Button* o = num_one_ = new Fl_Button(2,208,12,12,"1");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"1");
-				}
-				{
-					Fl_Button* o = num_eight_ = new Fl_Button(14,184,12,12,"8");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"8");
-				}
-				{
-					Fl_Button* o = num_five_ = new Fl_Button(14,196,12,12,"5");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"5");
-				}
-				{
-					Fl_Button* o = num_two_ = new Fl_Button(14,208,12,12,"2");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"2");
-				}
-				{
-					Fl_Button* o = num_nine_ = new Fl_Button(26,184,12,12,"9");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"9");
-				}
-				{
-					Fl_Button* o = num_six_ = new Fl_Button(26,196,12,12,"6");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"6");
-				}
-				{
-					Fl_Button* o = num_three_ = new Fl_Button(26,208,12,12,"3");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"3");
-				}
-				{
-					Fl_Button* o = num_plus_ = new Fl_Button(38,184,12,12,"+");
-                	o->labelfont(1);
-                	o->labelsize(10);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"+");
-				}
-				{
-					Fl_Button* o = num_minus_ = new Fl_Button(38,196,12,12,"-");
-                	o->labelfont(1);
-                	o->labelsize(10);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"-");
-				}
-				{
-					Fl_Button* o = num_zero_ = new Fl_Button(38,208,12,12,"0");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"0");
-				}
-				{
-					Fl_Button* o = num_times_ = new Fl_Button(50,184,12,12,"*");
-                	o->labelfont(1);
-                	o->labelsize(10);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"*");
-				}
-				{
-					Fl_Button* o = num_divide_ = new Fl_Button(50,196,12,12,"/");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"/");
-				}
-				{
-					Fl_Button* o = num_point_ = new Fl_Button(50,208,12,12,".");
-                	o->labelfont(1);
-                	o->labelsize(12);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)".");
-				}
-				{
-					Fl_Button* o = num_power_ = new Fl_Button(62,184,12,12,"^");
-                	o->labelfont(1);
-                	o->labelsize(10);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"^");
-				}
-				{
-					Fl_Button* o = num_space_ = new Fl_Button(62,196,12,12," ");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)" ");
-				}
-				{
-					Fl_Button* o = num_comma_ = new Fl_Button(62,208,12,12,",");
-                	o->labelfont(1);
-                	o->labelsize(12);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)",");
-				}
-				{
-					Fl_Button* o = num_leftpar_ = new Fl_Button(74,184,12,12,"(");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"(");
-				}
-				{
-					Fl_Button* o = num_leftbrace_ = new Fl_Button(74,196,12,12,"{");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"{");
-				}
-				{
-					Fl_Button* o = num_colon_ = new Fl_Button(74,208,12,12,":");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)":");
-				}
-				{
-					Fl_Button* o = num_rightpar_ = new Fl_Button(86,184,12,12,")");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)")");
-				}
-				{
-					Fl_Button* o = num_rightbrace_ = new Fl_Button(86,196,12,12,"}");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"}");
-				}
-				{
-					Fl_Button* o = num_equal_ = new Fl_Button(86,208,12,12,"=");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"=");
-				}
-				{
-					Fl_Button* o = num_x_ = new Fl_Button(98,184,12,12,"x");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"x");
-				}
-				{
-					Fl_Button* o = num_a_ = new Fl_Button(98,196,12,12,"a");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"a");
-				}
-				{
-					Fl_Button* o = num_i_ = new Fl_Button(98,208,12,12,"i");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"i");
-				}
-				{
-					Fl_Button* o = num_y_ = new Fl_Button(110,184,12,12,"y");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"y");
-				}
-				{
-					Fl_Button* o = num_b_ = new Fl_Button(110,196,12,12,"b");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"b");
-				}
-				{
-					Fl_Button* o = num_j_ = new Fl_Button(110,208,12,12,"j");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"j");
-				}
-				{
-					Fl_Button* o = num_z_ = new Fl_Button(122,184,12,12,"z");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"z");
-				}
-				{
-					Fl_Button* o = num_c_ = new Fl_Button(122,196,12,12,"c");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"c");
-				}
-				{
-					Fl_Button* o = num_k_ = new Fl_Button(122,208,12,12,"k");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"k");
-				}
-				{
-					Fl_Button* o = num_f_ = new Fl_Button(134,184,12,12,"f");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"f");
-				}
-				{
-					Fl_Button* o = num_g_ = new Fl_Button(134,196,12,12,"%");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"%");
-				}
-				{
-					Fl_Button* o = num_t_ = new Fl_Button(134,208,12,12,"t");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"t");
-				}
-				{
-					Fl_Button* o = num_enter_ = new Fl_Button(146,184,12,12,"@#<-");
-                	o->labeltype(FL_SYMBOL_LABEL);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_backspace);
-				}
-				{
-					Fl_Button* o = num_help_ = new Fl_Button(146,196,12,12,"?");
-                	o->labelfont(1);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_insert,(void*)"?");
-				}
-				{
-					Fl_Button* o = num_back_ = new Fl_Button(146,208,12,12,"@returnarrow");
-                	o->labeltype(FL_SYMBOL_LABEL);
-                	o->labelsize(9);
-                	o->callback((Fl_Callback*)cb_menu_enter);
-				}
+              console = new FltkConsole(2,16,156,153,9);
+            }
+            {
+              menubar = new Fl_Menu_Bar(2, 169, 156, 15);
+              menubar->menu(menuitems);
+              menubar->textfont(8);
+              menubar->textsize(10);
+            }
+            {
+              Fl_Button* o = num_seven_ = new Fl_Button(2,184,12,12,"7");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"7");
+            }
+            {
+              Fl_Button* o = num_four_ = new Fl_Button(2,196,12,12,"4");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"4");
+            }
+            {
+              Fl_Button* o = num_one_ = new Fl_Button(2,208,12,12,"1");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"1");
+            }
+            {
+              Fl_Button* o = num_eight_ = new Fl_Button(14,184,12,12,"8");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"8");
+            }
+            {
+              Fl_Button* o = num_five_ = new Fl_Button(14,196,12,12,"5");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"5");
+            }
+            {
+              Fl_Button* o = num_two_ = new Fl_Button(14,208,12,12,"2");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"2");
+            }
+            {
+              Fl_Button* o = num_nine_ = new Fl_Button(26,184,12,12,"9");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"9");
+            }
+            {
+              Fl_Button* o = num_six_ = new Fl_Button(26,196,12,12,"6");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"6");
+            }
+            {
+              Fl_Button* o = num_three_ = new Fl_Button(26,208,12,12,"3");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"3");
+            }
+            {
+              Fl_Button* o = num_plus_ = new Fl_Button(38,184,12,12,"+");
+                      o->labelfont(1);
+                      o->labelsize(10);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"+");
+            }
+            {
+              Fl_Button* o = num_minus_ = new Fl_Button(38,196,12,12,"-");
+                      o->labelfont(1);
+                      o->labelsize(10);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"-");
+            }
+            {
+              Fl_Button* o = num_zero_ = new Fl_Button(38,208,12,12,"0");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"0");
+            }
+            {
+              Fl_Button* o = num_times_ = new Fl_Button(50,184,12,12,"*");
+                      o->labelfont(1);
+                      o->labelsize(10);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"*");
+            }
+            {
+              Fl_Button* o = num_divide_ = new Fl_Button(50,196,12,12,"/");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"/");
+            }
+            {
+              Fl_Button* o = num_point_ = new Fl_Button(50,208,12,12,".");
+                      o->labelfont(1);
+                      o->labelsize(12);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)".");
+            }
+            {
+              Fl_Button* o = num_power_ = new Fl_Button(62,184,12,12,"^");
+                      o->labelfont(1);
+                      o->labelsize(10);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"^");
+            }
+            {
+              Fl_Button* o = num_space_ = new Fl_Button(62,196,12,12," ");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)" ");
+            }
+            {
+              Fl_Button* o = num_comma_ = new Fl_Button(62,208,12,12,",");
+                      o->labelfont(1);
+                      o->labelsize(12);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)",");
+            }
+            {
+              Fl_Button* o = num_leftpar_ = new Fl_Button(74,184,12,12,"(");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"(");
+            }
+            {
+              Fl_Button* o = num_leftbrace_ = new Fl_Button(74,196,12,12,"{");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"{");
+            }
+            {
+              Fl_Button* o = num_colon_ = new Fl_Button(74,208,12,12,":");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)":");
+            }
+            {
+              Fl_Button* o = num_rightpar_ = new Fl_Button(86,184,12,12,")");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)")");
+            }
+            {
+              Fl_Button* o = num_rightbrace_ = new Fl_Button(86,196,12,12,"}");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"}");
+            }
+            {
+              Fl_Button* o = num_equal_ = new Fl_Button(86,208,12,12,"=");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"=");
+            }
+            {
+              Fl_Button* o = num_x_ = new Fl_Button(98,184,12,12,"x");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"x");
+            }
+            {
+              Fl_Button* o = num_a_ = new Fl_Button(98,196,12,12,"a");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"a");
+            }
+            {
+              Fl_Button* o = num_i_ = new Fl_Button(98,208,12,12,"i");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"i");
+            }
+            {
+              Fl_Button* o = num_y_ = new Fl_Button(110,184,12,12,"y");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"y");
+            }
+            {
+              Fl_Button* o = num_b_ = new Fl_Button(110,196,12,12,"b");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"b");
+            }
+            {
+              Fl_Button* o = num_j_ = new Fl_Button(110,208,12,12,"j");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"j");
+            }
+            {
+              Fl_Button* o = num_z_ = new Fl_Button(122,184,12,12,"z");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"z");
+            }
+            {
+              Fl_Button* o = num_c_ = new Fl_Button(122,196,12,12,"c");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"c");
+            }
+            {
+              Fl_Button* o = num_k_ = new Fl_Button(122,208,12,12,"k");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"k");
+            }
+            {
+              Fl_Button* o = num_f_ = new Fl_Button(134,184,12,12,"f");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"f");
+            }
+            {
+              Fl_Button* o = num_g_ = new Fl_Button(134,196,12,12,"%");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"%");
+            }
+            {
+              Fl_Button* o = num_t_ = new Fl_Button(134,208,12,12,"t");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"t");
+            }
+            {
+              Fl_Button* o = num_enter_ = new Fl_Button(146,184,12,12,"@#<-");
+                      o->labeltype(FL_SYMBOL_LABEL);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_backspace);
+            }
+            {
+              Fl_Button* o = num_help_ = new Fl_Button(146,196,12,12,"?");
+                      o->labelfont(1);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_insert,(void*)"?");
+            }
+            {
+              Fl_Button* o = num_back_ = new Fl_Button(146,208,12,12,"@returnarrow");
+                      o->labeltype(FL_SYMBOL_LABEL);
+                      o->labelsize(9);
+                      o->callback((Fl_Callback*)cb_menu_enter);
+            }
 
                 o->labelsize(10);
                 o->end();
@@ -843,21 +839,19 @@ int main(int argc, char **argv)
             {
                 Fl_Group* o = grapher = new Fl_Group(2, 15, 156, 215, "Graph");
                 drawing = new Drawer(2,16,156,203);
-      		    o->labelsize(10);
+      		      o->labelsize(10);
                 o->end();
             }
 
             {
                 Fl_Group* o = new Fl_Group(2, 15, 156, 215, "Edit");
                 editor_add_items(2,16,156,203,9);
-				o->labelsize(10);
+				        o->labelsize(10);
                 o->end();
             }
 
             {
                 Fl_Group* o = new Fl_Group(2, 15, 156, 215, "?");
-
-
                 {
                     HelpView* o = helpview_ = new HelpView(2, 16, 156, 181);
                     helpview_->textsize(8);
@@ -898,7 +892,6 @@ int main(int argc, char **argv)
                 }
                 o->labelsize(10);
                 o->end();
-
             }
 			
             init_editor();
@@ -917,8 +910,9 @@ int main(int argc, char **argv)
         o->end();
     }
 
+    w->end();
     RestartYacas();
-	w->callback(quit_cb);
+  	w->callback(quit_cb);
     w->show(argc, argv);
     atexit(myexit);
     return Fl::run();
