@@ -6,6 +6,7 @@
  *
  */
 
+//#include <stdio.h>
 
 #include "yacasprivate.h"
 #include "anumber.h"
@@ -23,8 +24,40 @@ LispBoolean BaseGreaterThan(ANumber& a1, ANumber& a2);
 LispBoolean BaseLessThan(ANumber& a1, ANumber& a2);
 void BaseSqrt(ANumber& aResult, ANumber& N);
 
+// The default is 8, but it is suspected mose numbers will be short integers that fit into
+// one or two words. For these numbers memory allocation will be a lot more friendly.
+#define NUMBER_GRANULARITY 2 
 
 
+#ifdef HAVE_STDIO_H 
+#include <stdio.h>
+#endif
+void PrintNumber(char* prefix,ANumber& aNumber)
+{
+#ifdef HAVE_STDIO_H 
+  printf("%s\n",prefix);
+  printf("%d words, %d after point (x10^%d), 10-prec. %d\n",
+        aNumber.NrItems(),aNumber.iExp, aNumber.iTensExp,aNumber.iPrecision);
+  int i;
+  for (i=aNumber.NrItems()-1;i>=0;i--)
+  {
+    if (aNumber.iExp == i+1) printf(".\n");
+    PlatWord w = (aNumber)[i];
+    PlatWord bit = (WordBase)>>1;
+    int k=0;
+    while (bit)
+    {
+      if ((k&3)==0) printf(" ");
+      k++;
+      if (w&bit) printf("1");
+      else printf("0");
+      bit>>=1;
+    }
+    printf("\n");
+  }
+#endif
+}
+
 static LispInt DigitIndex(LispInt c)
 {
     if (c>='0' && c<='9')
@@ -53,7 +86,7 @@ ANumber::~ANumber()
 {
 }
 
-ANumber::ANumber(LispInt aPrecision)
+ANumber::ANumber(LispInt aPrecision) : CArrayGrower<PlatWord>(NUMBER_GRANULARITY)
 {
     LISPASSERT(sizeof(PlatDoubleWord) >= 2*sizeof(PlatWord));
     iPrecision = aPrecision;
@@ -64,7 +97,7 @@ ANumber::ANumber(LispInt aPrecision)
 }
 
 /* Allow use of external arrays */
-ANumber::ANumber(PlatWord *aArray, LispInt aSize, LispInt aPrecision)
+ANumber::ANumber(PlatWord *aArray, LispInt aSize, LispInt aPrecision): CArrayGrower<PlatWord>(NUMBER_GRANULARITY)
 {
     LISPASSERT(sizeof(PlatDoubleWord) >= 2*sizeof(PlatWord));
     iPrecision = aPrecision;
@@ -75,7 +108,7 @@ ANumber::ANumber(PlatWord *aArray, LispInt aSize, LispInt aPrecision)
 }
 
 /* ANumber: Constructor for an arbitrary precision number. */
-ANumber::ANumber(LispCharPtr aString,LispInt aPrecision,LispInt aBase)
+ANumber::ANumber(const LispCharPtr aString,LispInt aPrecision,LispInt aBase): CArrayGrower<PlatWord>(NUMBER_GRANULARITY)
 {
     SetPrecision(aPrecision);
     SetTo(aString,aBase);
@@ -108,7 +141,7 @@ void IntToAscii(LispString& aString,PlatDoubleWord aInt, LispInt aBase)
     IntToBaseString(aString,aInt,aBase);
     LispInt i;
     LispInt nr = aString.NrItems();
-    for (i=0;i<nr>>1;i++)
+    for (i=0;i<(nr>>1);i++)
     {
         LispChar c = aString[i];
         aString[i] = Digit(aString[nr-i-1]);
@@ -128,6 +161,7 @@ void IntToAscii(LispString& aString,PlatDoubleWord aInt, LispInt aBase)
 
 LispInt WordDigits(LispInt aPrecision, LispInt aBase)
 {
+    if (aPrecision == 0) return 0;
     LispInt bitsPerBase=0;
 
     while (aBase!=0)
@@ -135,11 +169,17 @@ LispInt WordDigits(LispInt aPrecision, LispInt aBase)
         aBase>>=1;
         bitsPerBase++;
     }
+    // I changed this to add two extra words at the end in stead of one, when
+    // we moved over to scientific notation. An example of what went wrong was
+    // typing -6.23, which for sufficiently low precision got read as 6.229999
+    // The original thought was that one word should be enough. This  will have
+    // to be examined more closely.
+//    return (aPrecision*bitsPerBase+2*WordBits)/WordBits;
     return (aPrecision*bitsPerBase+WordBits)/WordBits;
 }
 
 
-void ANumber::SetTo(LispCharPtr aString,LispInt aBase)
+void ANumber::SetTo(const LispCharPtr aString,LispInt aBase)
 {
     SetNrItems(0);
     
@@ -149,7 +189,7 @@ void ANumber::SetTo(LispCharPtr aString,LispInt aBase)
     iExp = 0;
     iTensExp = 0;
     
-    LispCharPtr endptr = aString;
+    const LispCharPtr endptr = aString;
 
     // Parse minus sign
     if (*endptr == '-')
@@ -165,7 +205,7 @@ void ANumber::SetTo(LispCharPtr aString,LispInt aBase)
     {
         if (aString[endNumberIndex]=='.')
             endIntIndex = endNumberIndex;
-        if (aString[endNumberIndex]=='e' || aString[endNumberIndex]=='E')
+        if (aBase < 14 && aString[endNumberIndex]=='e' || aString[endNumberIndex]=='E')
             endFloatIndex = endNumberIndex;
         endNumberIndex++;
     }
@@ -176,9 +216,18 @@ void ANumber::SetTo(LispCharPtr aString,LispInt aBase)
         endIntIndex = endFloatIndex;
     if (endFloatIndex == endIntIndex+1)
         endFloatIndex = endIntIndex;
+//printf("%s\n",aString);        
+//printf("\tendint = %d, endfloat = %d, endnumber = %d\n",endIntIndex,endFloatIndex,endNumberIndex);        
+
+//printf("%d digits\n",endFloatIndex-endIntIndex-1);
+
+    if (endFloatIndex-endIntIndex-1 > iPrecision) 
+    {
+      iPrecision = endFloatIndex-endIntIndex-1;
+    }
         
     // Go to least significant digit first
-    LispCharPtr ptr = aString + endIntIndex-1; 
+    const LispCharPtr ptr = aString + endIntIndex-1; 
     
     // Now parse the integer part of the number.
     ANumber factor2(iPrecision);
@@ -187,28 +236,29 @@ void ANumber::SetTo(LispCharPtr aString,LispInt aBase)
     {
         ANumber term(iPrecision);
         term.CopyFrom(factor2);
-        BaseTimesInt(term, DigitIndex(*ptr), WordBase);
-        BaseAdd(*this,term,WordBase);
+        WordBaseTimesInt(term, DigitIndex(*ptr));
+        WordBaseAdd(*this,term);
         /*
         ANumber current(iPrecision);
         current.CopyFrom(*this);
         BaseAddFull(*this,current,term);
         */
-        BaseTimesInt(factor2, aBase, WordBase);
+        WordBaseTimesInt(factor2, aBase);
         ptr--;
     }
 
     //Parse the fraction
     if (endFloatIndex > endIntIndex)
     {
-        LispString fraction(&aString[endIntIndex+1]);
+        LispString fraction((LispCharPtr)&aString[endIntIndex+1]);
         LispInt i;
 
         // Map to a char base number
-        LispInt nr = fraction.NrItems()-1; //Excluding the zero terminator
+        LispInt nr;// = fraction.NrItems()-1; //Excluding the zero terminator
+        nr = endFloatIndex - endIntIndex-1;
         LispString::ElementTypePtr fractionPtr = &fraction[0];
 
-        for (i=0;i<nr>>1;i++)
+        for (i=0;i<(nr>>1);i++)
         {
             LispChar c = fractionPtr[i];
             fractionPtr[i] = DigitIndex(fractionPtr[nr-i-1]);
@@ -220,7 +270,9 @@ void ANumber::SetTo(LispCharPtr aString,LispInt aBase)
         LispString base;
         IntToBaseString(base,WordBase,aBase);
 
-        LispInt nrDigits = WordDigits(iPrecision,aBase);
+        LispInt nrDigits;
+        nrDigits = WordDigits(iPrecision,aBase)/*+1*/;        
+
         for (i=0;i<nrDigits;i++)
         {
             PlatWord word=0;
@@ -232,15 +284,7 @@ void ANumber::SetTo(LispCharPtr aString,LispInt aBase)
                 LispInt nrc=fraction.NrItems();
                 copied.GrowTo(nrc);
                 copied.SetNrItems(nrc);
-                /*TODO remove?
-                typename LispString::ElementTypePtr fractionPtr = &fraction[0];
-                typename LispString::ElementTypePtr copiedPtr = &copied[0];
-                */
                 PlatMemCopy(&copied[0],  &fraction[0], nrc*sizeof(LispString::ElementType));
-                /*TODO remove?
-                 for (j=0;j<nrc;j++)
-                     copiedPtr[j]=fractionPtr[j];
-                 */
             }
             BaseMultiply(fraction, copied, base, aBase);
 
@@ -258,16 +302,23 @@ void ANumber::SetTo(LispCharPtr aString,LispInt aBase)
             Insert(0,word);
             iExp++;
         }
+//        Delete(0);
+//        iExp--;
     }
 
     // Parse the E<num> part at the end
     if (endNumberIndex > endFloatIndex+1)
     {
-        iTensExp = PlatAsciiToInt(&aString[endFloatIndex+1]);
+      if (aString[endFloatIndex] == '.') endFloatIndex++;
+        iTensExp = PlatAsciiToInt((LispCharPtr)&aString[endFloatIndex+1]);
+//printf("%s mapped to %d\n",&aString[endFloatIndex+1],iTensExp);
     }
+
+    DropTrailZeroes();
+//PrintNumber("      ",*this);
 }
 
-void ANumber::CopyFrom(ANumber& aOther)
+void ANumber::CopyFrom(const ANumber& aOther)
 {
     iExp       = aOther.iExp;
     iTensExp   = aOther.iTensExp;
@@ -278,15 +329,50 @@ void ANumber::CopyFrom(ANumber& aOther)
 
     //TODO there HAS to be a faster way to copy...
     LispInt nr = aOther.NrItems();
+    if (nr)
+    {
+//this is actually slower!      PlatMemCopy((LispCharPtr)&((*this)[0]),(LispCharPtr)&( aOther[0]),nr*sizeof(ANumber::ElementType));
+
+      ANumber::ElementTypePtr sptr = &( aOther[0]);
+      ANumber::ElementTypePtr tptr = &((*this)[0]);
+      while (nr--)
+      {
+          *tptr++ = *sptr++;
+      }
+    }
+    else
+    {
+      GrowTo(1);
+      SetNrItems(1);
+      (*this)[0] = 0;
+    }
+}
+
+
+
+LispBoolean ANumber::ExactlyEqual(const ANumber& aOther)
+{
+  if (iExp       != aOther.iExp) return LispFalse;
+  if (iTensExp   != aOther.iTensExp) return LispFalse;
+  if (iNegative  != aOther.iNegative) return LispFalse;
+//  if (iPrecision != aOther.iPrecision) return LispFalse;
+
+  //TODO there HAS to be a faster way to copy...
+  LispInt nr = NrItems();
+  if (nr)
+  {
     ANumber::ElementTypePtr sptr = &( aOther[0]);
     ANumber::ElementTypePtr tptr = &((*this)[0]);
     while (nr--)
     {
-        *tptr++ = *sptr++;
+      if (*tptr++ != *sptr++) return LispFalse;
     }
+  }
+  return LispTrue;
 }
 
 
+
 /* Negate negates a number. */
 void Negate(ANumber& aNumber)
 {
@@ -299,30 +385,11 @@ void Negate(ANumber& aNumber)
 void Multiply(ANumber& aResult, ANumber& a1, ANumber& a2)
 {
     // Truncate zeroes (the multiplication is heavy enough as it is...)
-    LispInt todel;
-
-    todel=0;
-    while (todel<a1.iExp && a1[todel] == 0)
-    {
-        todel++;
-    }
-    if (todel)
-    {
-        a1.iExp-=todel;
-        a1.Delete(0,todel);
-    }
-
-    todel=0;
-    while (todel<a2.iExp && a2[todel] == 0)
-    {
-        todel++;
-    }
-    if (todel)
-    {
-        a2.iExp-=todel;
-        a2.Delete(0,todel);
-    }
+    a1.DropTrailZeroes();
+    a2.DropTrailZeroes();
 
+    // this does some additional removing, as for the multiplication we don't need
+    // any trailing zeroes at all, regardless of the value of iExp
     LispInt end;
 
     end=a1.NrItems();
@@ -338,10 +405,13 @@ void Multiply(ANumber& aResult, ANumber& a1, ANumber& a2)
         end--;
     }
     a2.SetNrItems(end);
-    
+
     // Multiply
     BaseMultiplyFull(aResult,a1,a2);
 
+//PrintNumber("Mult",aResult);
+
+
     // Adjust the sign
     if (IsPositive(a1) && IsPositive(a2))
         aResult.iNegative = LispFalse;
@@ -360,12 +430,20 @@ void Multiply(ANumber& aResult, ANumber& a1, ANumber& a2)
         a2.Append(0);
     while(aResult.NrItems()<aResult.iExp+1)
         aResult.Append(0);
+
+
+    aResult.DropTrailZeroes();
 }
 
 static void BalanceFractions(ANumber& a1, ANumber& a2)
 {
     PlatWord word=0;
 
+//ANumber a3("10e2");
+//PrintNumber("a3 enter ",a3);
+//a3.iTensExp = 0;
+//PrintNumber("a3 enter ",a3);
+
     LispInt nr;
 
     nr = a2.iExp - a1.iExp;
@@ -382,6 +460,29 @@ static void BalanceFractions(ANumber& a1, ANumber& a2)
         a2.Insert(0,word,nr);
         a2.iExp+=nr;
     }
+
+
+    //TODO this is not the fastest way to multiply by 10^exp
+    if (a1.iTensExp < a2.iTensExp)
+    {
+      LispInt diff = a2.iTensExp - a1.iTensExp;
+      a2.iTensExp = a1.iTensExp;
+      while (diff > 0)
+      {
+        WordBaseTimesInt(a2,10);
+        diff--;
+      }
+    }
+    else if (a2.iTensExp < a1.iTensExp)
+    {
+      LispInt diff = a1.iTensExp - a2.iTensExp;
+      a1.iTensExp = a2.iTensExp;
+      while (diff > 0)
+      {
+        WordBaseTimesInt(a1,10);
+        diff--;
+      }
+    }
 }
 
 void Add(ANumber& aResult, ANumber& a1, ANumber& a2)
@@ -441,6 +542,7 @@ void Add(ANumber& aResult, ANumber& a1, ANumber& a2)
             aResult.CopyFrom(zero);
         }
     }
+    aResult.DropTrailZeroes();
 }
 
 
@@ -498,6 +600,7 @@ void Subtract(ANumber& aResult, ANumber& a1, ANumber& a2)
             aResult.CopyFrom(zero);
         }
     }
+    aResult.DropTrailZeroes();
 }
 
 
@@ -528,12 +631,14 @@ LispBoolean LessThan(ANumber& a1, ANumber& a2)
 }
 
 
-void  ANumberToString(LispString& aResult, ANumber& aNumber, LispInt aBase)
+void  ANumberToString(LispString& aResult, ANumber& aNumber, LispInt aBase, LispBoolean aForceFloat)
 {
     LispInt nr = aNumber.NrItems();
     while (nr>1 && aNumber[nr-1] == 0) nr--;
     aNumber.SetNrItems(nr);
 
+    LispInt tensExp = aNumber.iTensExp;
+
     //Formatting small numbers can be done faster.
     if (aNumber.iExp == 0 && nr == 1)
     {
@@ -543,7 +648,7 @@ void  ANumberToString(LispString& aResult, ANumber& aNumber, LispInt aBase)
         {
             LispInt i;
             LispString::ElementTypePtr rptr = &aResult[0];
-            for (i=0;i<nr>>1;i++)
+            for (i=0;i<(nr>>1);i++)
             {
                 LispString::ElementType c=rptr[i];
                 rptr[i] = Digit(rptr[nr-i-1]);
@@ -553,6 +658,11 @@ void  ANumberToString(LispString& aResult, ANumber& aNumber, LispInt aBase)
                 rptr[nr>>1] = Digit(rptr[nr>>1]);
         }
 
+        if (aForceFloat)
+        {
+          if (!(aResult.NrItems()==1 && aResult[0] == '0'))
+            aResult.Append('.');
+        }
         if (aNumber.iNegative)
         {
             if (aResult.NrItems()>1 || aResult[0] != '0')
@@ -561,6 +671,7 @@ void  ANumberToString(LispString& aResult, ANumber& aNumber, LispInt aBase)
                 aResult.Insert(0,c);
             }
         }
+        
         goto TENSEXP;
     }
     {
@@ -614,7 +725,7 @@ void  ANumberToString(LispString& aResult, ANumber& aNumber, LispInt aBase)
         // swap order of the digits, and map to ascii
         {
             LispString::ElementTypePtr rptr = &aResult[0];
-            for (i=0;i<nr>>1;i++)
+            for (i=0;i<(nr>>1);i++)
             {
                 LispString::ElementType c=rptr[i];
                 rptr[i] = rptr[nr-i-1];
@@ -626,7 +737,7 @@ void  ANumberToString(LispString& aResult, ANumber& aNumber, LispInt aBase)
         while(number.NrItems()<number.iExp)
             number.Append(0);
         number.SetNrItems(number.iExp);
-        if (number.iExp > 0 && !IsZero(number))
+        if (aForceFloat || (number.iExp > 0 && !IsZero(number)))
         {
             LispInt digitPos = aResult.NrItems();
 
@@ -634,7 +745,7 @@ void  ANumberToString(LispString& aResult, ANumber& aNumber, LispInt aBase)
             // Build the fraction
             for (i=0;i<number.iPrecision+1;i++)
             {
-                BaseTimesInt(number, aBase, WordBase);
+                WordBaseTimesInt(number, aBase);
                 if (number.NrItems() > number.iExp)
                 {
                     aResult.Append((LispChar)(number[number.iExp]));
@@ -665,7 +776,6 @@ void  ANumberToString(LispString& aResult, ANumber& aNumber, LispInt aBase)
                 }
             }
             aResult.SetNrItems(aResult.NrItems()-1);
-
             // Insert dot
             LispChar c='.';
             aResult.Insert(digitPos,c);
@@ -676,7 +786,7 @@ void  ANumberToString(LispString& aResult, ANumber& aNumber, LispInt aBase)
             {
                 nr--;
             }
-            if (aResult[nr-1] == '.')
+            if (aResult[nr-1] == '.' && nr == 2 && aResult[0] == 0)
                 nr--;
             aResult.SetNrItems(nr);
         }
@@ -703,12 +813,19 @@ void  ANumberToString(LispString& aResult, ANumber& aNumber, LispInt aBase)
 
     //Handle tens exp
 TENSEXP:
-    if (aNumber.iTensExp != 0)
+    if (tensExp != 0 && 
+        !(aResult[0] == '0' && aResult.NrItems() == 1))
     {
-        aResult.Append('E');
+        aResult.Append('e');
         LispString tens;
         //hier
-        IntToAscii(tens,aNumber.iTensExp, 10);
+        LispInt tenex = tensExp;
+        if (tenex<0)
+        {
+          aResult.Append('-');
+          tenex = -tenex;
+        }
+        IntToAscii(tens,tenex, 10);
         LispInt i,nr;
         nr=PlatStrLen(&tens[0]);
         for (i=0;i<nr;i++)
@@ -724,7 +841,7 @@ void BaseAddFull(ANumber& aResult, ANumber& a1, ANumber& a2)
 {
     // Initialize result
     aResult.CopyFrom(a1);
-    BaseAdd(aResult,a2,WordBase);
+    WordBaseAdd(aResult,a2);
 }
 
 
@@ -785,7 +902,7 @@ void BaseSubtract(ANumber& aResult, ANumber& a2, LispInt offset)
 void BaseMultiplyFull(ANumber& aResult, ANumber& a1, ANumber& a2)
 {
     // Initialize result
-    BaseMultiply(aResult,a1,a2,WordBase);
+    WordBaseMultiply(aResult,a1,a2);
 }
 
 
@@ -852,7 +969,7 @@ void BaseShiftRight(ANumber& a, LispInt aNrBits)
     LispInt residue = aNrBits % WordBits;
 
     // Bit mask: bits that are going to be shifted out of each word.
-    LispInt bitMask = (1L<<residue)-1;
+    PlatDoubleWord bitMask = (PlatDoubleWord)((((PlatDoubleWord)1)<<residue)-1);
 
     // Nr of bits to move to the other side.
     LispInt otherSideBits = WordBits-residue;
@@ -862,16 +979,22 @@ void BaseShiftRight(ANumber& a, LispInt aNrBits)
     LispInt nr = a.NrItems();
 
     ANumber::ElementTypePtr ptr = &a[0];
-
-    for (i=0;i<nr-wordsShifted;i++)
+    ANumber::ElementTypePtr ptrshifted = &a[wordsShifted];
+    ANumber::ElementTypePtr endp = ptr +nr - wordsShifted;
+    if (ptr<endp)
     {
-        PlatDoubleWord newCarry =
-            (((PlatDoubleWord)ptr[i+wordsShifted]) & bitMask)<<otherSideBits;
-
-        ptr[i] = (ptr[i+wordsShifted]>>residue);
-
-        if (i > 0)
-            ptr[i-1] |= newCarry;
+      *ptr = ((*ptrshifted)>>residue);
+      ptr++;
+      ptrshifted++;
+      while (ptr<endp)
+  //    for (i=1;i<nr-wordsShifted;i++)
+      {
+        PlatDoubleWord newCarry = (((PlatDoubleWord)*ptrshifted) & bitMask)<<otherSideBits;
+        *ptr = ((*ptrshifted)>>residue);
+        ptr[-1] |= newCarry;
+        ptr++;
+        ptrshifted++;
+      }
     }
 
     int start=nr-wordsShifted;
@@ -879,20 +1002,10 @@ void BaseShiftRight(ANumber& a, LispInt aNrBits)
         start=0;
     for (i=start;i<nr;i++)
     {
-        ptr[i] = 0;
+        a[i] = 0;
     }
 }
 
-
-
-
-
-
-
-
-
-
-
 void BaseShiftLeft(ANumber& a, LispInt aNrBits)
 {
     // Number of words a word jumps
@@ -937,22 +1050,29 @@ void BaseShiftLeft(ANumber& a, LispInt aNrBits)
 /* Binary Greatest common divisor algorithm. */
 void BaseGcd(ANumber& aResult, ANumber& a1, ANumber& a2)
 {
-    a1.iNegative = a2.iNegative = LispFalse;
     ANumber zero("0",Precision(aResult),10);
     ANumber u("0",Precision(aResult));
     ANumber v("0",Precision(aResult));
     u.CopyFrom(a1);
     v.CopyFrom(a2);
+    u.iNegative = v.iNegative = LispFalse;
 
      LispInt k=0;
-    
-    while (IsEven(u) && IsEven(v))
-    {
-        BaseShiftRight(u,1);
-        BaseShiftRight(v,1);
-        k++;
-    }
 
+
+    {
+      LispInt i=0;
+      PlatWord bit=1;
+      while (u[i] == 0 && v[i]==0) i++;
+      k+=WordBits*i;
+      while ((u[i]&bit) == 0 && (v[i]&bit)==0) 
+      {
+        bit<<=1;
+        k++;
+      }
+      BaseShiftRight(u,k);
+      BaseShiftRight(v,k);
+    }
     ANumber t("0",10);
 
     if (IsOdd(u))
@@ -965,11 +1085,20 @@ void BaseGcd(ANumber& aResult, ANumber& a1, ANumber& a2)
 
     while (!IsZero(t))
     {
-        while (IsEven(t))
-        {
-            BaseShiftRight(t,1);
-        }
 
+        {
+          LispInt k=0;
+          LispInt i=0;
+          PlatWord bit=1;
+          while (t[i] == 0) i++;
+          k+=WordBits*i;
+          while ((t[i]&bit) == 0) 
+          {
+            bit<<=1;
+            k++;
+          }
+          BaseShiftRight(t,k);
+        }
         if (GreaterThan(t,zero))
         {
             u.CopyFrom(t);
@@ -982,6 +1111,7 @@ void BaseGcd(ANumber& aResult, ANumber& a1, ANumber& a2)
         Subtract(t,u,v);
      }
     aResult.CopyFrom(u);
+    aResult.iNegative=LispFalse;
     BaseShiftLeft(aResult,k);
 }
 
@@ -1007,8 +1137,8 @@ void BaseDivide(ANumber& aQuotient, ANumber& aRemainder, ANumber& a1, ANumber& a
     
     //D1:
     PlatDoubleWord d = WordBase/(a2[n-1]+1);
-    BaseTimesInt(a1, d, WordBase);
-    BaseTimesInt(a2, d, WordBase);
+    WordBaseTimesInt(a1, d);
+    WordBaseTimesInt(a2, d);
     a1.Append(0);
     a2.Append(0);
     
@@ -1033,7 +1163,7 @@ void BaseDivide(ANumber& aQuotient, ANumber& aRemainder, ANumber& a1, ANumber& a
         //D4:
         ANumber sub(Precision(aQuotient));
         sub.CopyFrom(a2);
-        BaseTimesInt(sub, q, WordBase);
+        WordBaseTimesInt(sub, q);
         sub.Append(0);
         
         PlatSignedDoubleWord carry;
@@ -1063,7 +1193,7 @@ void BaseDivide(ANumber& aQuotient, ANumber& aRemainder, ANumber& a1, ANumber& a
             {
                 q--;
                 sub.CopyFrom(a2);
-                BaseTimesInt(sub, q, WordBase);
+                WordBaseTimesInt(sub, q);
                 sub.Append(0);
             }
             
@@ -1080,7 +1210,7 @@ void BaseDivide(ANumber& aQuotient, ANumber& aRemainder, ANumber& a1, ANumber& a
                     word+=WordBase;
                     carry--;
                 }
-                a1[digit+j] = ((PlatWord)(word%WordBase));
+                a1[digit+j] = ((PlatWord)(word));
             }
         }
         LISPASSERT(carry == 0);
@@ -1157,7 +1287,7 @@ void IntegerDivide(ANumber& aQuotient, ANumber& aRemainder, ANumber& a1, ANumber
         aQuotient.iExp = a1.iExp-a2.iExp;
         aQuotient.iTensExp = a1.iTensExp-a2.iTensExp;
         // Divide the mantissas
-        BaseDivide(aQuotient, aRemainder, a1, a2,WordBase);
+        WordBaseDivide(aQuotient, aRemainder, a1, a2);
     }
 
     // Correct for signs
@@ -1208,7 +1338,7 @@ void BaseSqrt(ANumber& aResult, ANumber& N)
     ANumber v2 (Precision(aResult));
     ANumber uv2(Precision(aResult));
     ANumber n  (Precision(aResult));
-    ANumber two("2");
+    ANumber two("2",10);
 
     //sqrt(1) = 1, sqrt(0) = 0
     if( BaseGreaterThan(two, N))
@@ -1253,13 +1383,13 @@ void BaseSqrt(ANumber& aResult, ANumber& N)
 
         // n = (u+v)^2  =  u^2 + 2*u*v + v^2 = u2+uv2+v2
         n.CopyFrom(u2);
-        BaseAdd(n,uv2,WordBase);
-        BaseAdd(n,v2,WordBase);
+        WordBaseAdd(n,uv2);
+        WordBaseAdd(n,v2);
         // if n (possible new best estimate for sqrt(N)^2 is smaller than
         // N, then the bit l2 is set in the result, and add v to u.
         if( !BaseGreaterThan(n , N) )
         {
-            BaseAdd(u,v,WordBase); // u <- u+v
+            WordBaseAdd(u,v); // u <- u+v
             u2.CopyFrom(n);  // u^2 <- u^2 + 2*u*v + v^2
         }
     }
@@ -1268,18 +1398,25 @@ void BaseSqrt(ANumber& aResult, ANumber& N)
 
 void Sqrt(ANumber& aResult, ANumber& N)
 {
-    LispInt digs = WordDigits(aResult.iPrecision, 10);
+    LispInt digs = WordDigits(N.iPrecision, 10);
     PlatWord zero=0;
-    while(N.iExp<2*digs)
+
+   if ((N.iTensExp&1) != 0)
+    {
+      WordBaseTimesInt(N,10);
+      N.iTensExp--;
+    }
+    while(N.iExp<2*digs || (N.iExp&1))
     {
         N.Insert(0,zero);
         N.iExp++;
     }
+    LispInt resultDigits = N.iExp/2;
     
     BaseSqrt(aResult, N);
-    aResult.iExp=digs;
+    aResult.iExp=resultDigits;
 
-    //TODO!!!@@@###$$$ iTensExp???
+    aResult.iTensExp = (N.iTensExp/2);
 }
 
 
@@ -1308,20 +1445,82 @@ LispBoolean Significant(ANumber& a)
 }
 
 
+void ANumber::RoundBits(void)
+{
+  PlatWord* ptr = (PlatWord*)iArray;
+  if (*ptr < (WordBase/2))
+  {
+    *ptr = 0;
+  }
+  else
+  {
+    *ptr = 0;
+    PlatDoubleWord carry=1;
+    LispInt i,nr=NrItems();
+    for (i=1;i<nr;i++)
+    {
+        PlatDoubleWord dword = ptr[i]+carry;
+        ptr[i] = (PlatWord)dword;
+        carry = dword >> WordBits;
+    }
+    if (carry)
+    {
+      Append(carry);
+    }
+  }
+}
+
 void ANumber::ChangePrecision(LispInt aPrecision)
 {
+  //First, round.
+/*FIXME TODO not working correctly yet */
+  //TODO code bloat! Deserves its own routine!
+  if (aPrecision == 0 && iExp>1)
+  {
+    RoundBits();
+  }
+//  return;
+
+  //FIXME the following line is there to assure there are enough words. Somehow this got truncated?
+  //FIXME numerics.yts fails
+  while (iExp+1>NrItems()) Append(0);
+
   LispInt oldExp = iExp;
   
   iPrecision = aPrecision;
-  iExp = WordDigits(iPrecision,10);
-
-  if (iExp < oldExp)
+  LispInt newExp = WordDigits(iPrecision,10);
+  if (newExp < oldExp)
   {
+    iExp = newExp;
     Delete(0,oldExp-iExp);
   }
-  else if (iExp > oldExp)
+  else if (newExp > oldExp)
   {
+    iExp = newExp;
     PlatWord zero = 0;
-    Insert(0,zero,iExp-oldExp);
+    Insert(0,zero,newExp-oldExp);
   }
 }
+
+
+
+void ANumber::DropTrailZeroes()
+{
+  while (iExp+1>NrItems()) Append(0);
+  
+  {
+    LispInt nr=NrItems();
+    while (nr>iExp+1 && (*this)[nr-1] == 0) nr--;
+    SetNrItems(nr);
+  }
+  {
+    LispInt low=0;
+    while (low<iExp && (*this)[low] == 0) low++;
+    if (low)
+    {
+      Delete(0,low);
+      iExp-=low;
+    }
+  }
+}
+
