@@ -6,15 +6,15 @@
 #ifndef __choices_h__
 #define __choices_h__
 
-/** Turn on LISP_DEBUGHEAP if you want to see run-time statistics
+/** Turn on YACAS_DEBUG if you want to see run-time statistics
  * after typing Exit()
  */
-//#define LISP_DEBUGHEAP
+//#define YACAS_DEBUG
 
 /** Turn on USE_ASSERT to find programming errors through the asserts
  *  placed in various places of the application.
  */
-#define USE_ASSERT
+//#define USE_ASSERT
 
 /** Turn on NO_EXCEPTIONS if you want to disable run-time checking
  *  while executing commands.
@@ -28,6 +28,11 @@
  */
 //#define USE_KARATSUBA
 
+/** Enable long reference counts. This makes EVERY object 2 bytes
+ larger, so use only if needed.
+ */
+// #define USE_LONG_REF_COUNTS
+
 
 #endif
 
