@@ -8,11 +8,17 @@
 #include "standard.h"
 #include "lispuserfunc.h"
 #include "mathuserfunc.h"
+#include "errors.h"
+#include "archiver.h"
 
+// we need this only for digits_to_bits
+#include "numbers.h"
 
 #define InternalEval iEvaluator->Eval
 
-LispEnvironment::LispEnvironment(LispCommands& aCommands,
+
+LispEnvironment::LispEnvironment(
+                    YacasCoreCommands& aCoreCommands,
                     LispUserFunctions& aUserFunctions,
                     LispGlobal& aGlobals,
                     LispHashTable& aHashTable,
@@ -23,15 +29,25 @@ LispEnvironment::LispEnvironment(LispCommands& aCommands,
                     LispOperators &aPostFixOperators,
                     LispOperators &aBodiedOperators,
                     LispInput*    aCurrentInput)
-    : iPrecision(10),iEvalDepth(0),iMaxEvalDepth(1000),
-    iEvaluator(NEW BasicEvaluator),iSecure(0),iLastUniqueId(1),
-    iErrorOutput(iError),iCommands(aCommands),
+    : 
+    iPrecision(10),	// default user precision of 10 decimal digits
+	iBinaryPrecision(34),	// same as 34 bits
+    iEvalDepth(0),
+    iMaxEvalDepth(1000),
+    iArchive(NULL),
+    iEvaluator(NEW BasicEvaluator),
+    iSecure(0),
+    iLastUniqueId(1),
+    iErrorOutput(iError),
+    iDebugger(NULL),
+    iLocalsList(NULL),
+    iInitialOutput(aOutput),
+    iCoreCommands(aCoreCommands),
     iUserFunctions(aUserFunctions),
     iHashTable(aHashTable),
     iPrinter(aPrinter),
     iCurrentOutput(aOutput),
     iGlobals(aGlobals),
-    iLocalsList(NULL),
     iPreFixOperators(aPreFixOperators),
     iInFixOperators(aInFixOperators),
     iPostFixOperators(aPostFixOperators),
@@ -39,47 +55,23 @@ LispEnvironment::LispEnvironment(LispCommands& aCommands,
     iCurrentInput(aCurrentInput),
     theUserError(NULL),
     iPrettyPrinter(NULL),
-    iDebugger(NEW DefaultDebugger),
-    iCurrentTokenizer(&iDefaultTokenizer),
-    iArchive(NULL)
+    iCurrentTokenizer(&iDefaultTokenizer)
 {
-    iTrue=NULL;
-    iFalse=NULL;
+    iTrue = LispAtom::New(*this,"True");
+    iFalse = LispAtom::New(*this,"False");
 
-    iTrue = HashTable().LookUp("True");
-    iFalse = HashTable().LookUp("False");
-
-    iEndOfFile    = iHashTable.LookUp("EndOfFile");
-    iEndStatement = iHashTable.LookUp(";");
-    iProgOpen     = iHashTable.LookUp("[");
-    iProgClose    = iHashTable.LookUp("]");
-    iNth          = iHashTable.LookUp("Nth");
-    iBracketOpen  = iHashTable.LookUp("(");
-    iBracketClose = iHashTable.LookUp(")");
-    iListOpen     = iHashTable.LookUp("{");
-    iListClose    = iHashTable.LookUp("}");
-    iComma        = iHashTable.LookUp(",");
-    iList         = iHashTable.LookUp("List");
-    iProg         = iHashTable.LookUp("Prog");
-
-    iEndOfFile   ->IncreaseRefCount();
-    iEndStatement->IncreaseRefCount();
-    iProgOpen    ->IncreaseRefCount();
-    iProgClose   ->IncreaseRefCount();
-    iNth         ->IncreaseRefCount();
-    iBracketOpen ->IncreaseRefCount();
-    iBracketClose->IncreaseRefCount();
-    iListOpen    ->IncreaseRefCount();
-    iListClose   ->IncreaseRefCount();
-    iComma       ->IncreaseRefCount();
-    iList        ->IncreaseRefCount();
-    iProg        ->IncreaseRefCount();
-
-    iTrue        ->IncreaseRefCount();
-    iFalse       ->IncreaseRefCount();
-
-    iTrueAtom.Set(LispAtom::New(iTrue));
-    iFalseAtom.Set(LispAtom::New(iFalse));
+    iEndOfFile    = LispAtom::New(*this,"EndOfFile");
+    iEndStatement = LispAtom::New(*this,";");
+    iProgOpen     = LispAtom::New(*this,"[");
+    iProgClose    = LispAtom::New(*this,"]");
+    iNth          = LispAtom::New(*this,"Nth");
+    iBracketOpen  = LispAtom::New(*this,"(");
+    iBracketClose = LispAtom::New(*this,")");
+    iListOpen     = LispAtom::New(*this,"{");
+    iListClose    = LispAtom::New(*this,"}");
+    iComma        = LispAtom::New(*this,",");
+    iList         = LispAtom::New(*this,"List");
+    iProg         = LispAtom::New(*this,"Prog");
     PushLocalFrame(LispTrue);
     iCTokenizer.SetRemarkReceiver(*this);
 }
@@ -100,27 +92,17 @@ LispEnvironment::~LispEnvironment()
     }
     
     LISPASSERT(iLocalsList == NULL);
-    iTrue        ->DecreaseRefCount();
-    iFalse       ->DecreaseRefCount();
-
-    iEndOfFile   ->DecreaseRefCount();
-    iEndStatement->DecreaseRefCount();
-    iProgOpen    ->DecreaseRefCount();
-    iProgClose   ->DecreaseRefCount();
-    iNth         ->DecreaseRefCount();
-    iBracketOpen ->DecreaseRefCount();
-    iBracketClose->DecreaseRefCount();
-    iListOpen    ->DecreaseRefCount();
-    iListClose   ->DecreaseRefCount();
-    iComma       ->DecreaseRefCount();
-    iList        ->DecreaseRefCount();
-    iProg        ->DecreaseRefCount();
-
     delete iEvaluator;
-    delete iDebugger;
+    if (iDebugger) delete iDebugger;
     delete iArchive;
 }
 
+void LispEnvironment::SetPrecision(LispInt aPrecision)
+{
+    iPrecision = aPrecision;	// precision in decimal digits
+	iBinaryPrecision = digits_to_bits(aPrecision, BASE10);	// in bits
+}
+
 LispInt LispEnvironment::GetUniqueId()
 {
     return iLastUniqueId++;
@@ -231,6 +213,24 @@ void LispEnvironment::NewLocal(LispStringPtr aVariable,LispObject* aValue)
     iLocalsList->Add(NEW LispLocalVariable(aVariable, aValue));
 }
 
+void LispEnvironment::CurrentLocals(LispPtr& aResult)
+{
+  LispEnvironment::LocalVariableFrame* fr = iLocalsList;
+  LispEnvironment::LispLocalVariable* ptr = fr->iFirst;
+
+  LispEnvironment& aEnvironment = *this; //Pity, but we need this for the macros to work
+  LispObject* locals = NULL;
+  while (ptr != NULL)
+  {
+    locals = LA(ATOML(ptr->iVariable->String()))+LA(locals);
+//    printf("%s ",ptr->iVariable->String());
+    ptr = ptr->iNext;
+  }
+  aResult.Set(LIST(LA(ATOML("List")) + LA(locals)));  
+}
+
+
+
 LispPrinter& LispEnvironment::CurrentPrinter()
 {
     return iPrinter;
@@ -334,6 +334,13 @@ void LispEnvironment::DeclareRuleBase(LispStringPtr aOperator,
 {
     LispMultiUserFunction* multiUserFunc = MultiUserFunction(aOperator);
 
+    /*
+     if (multiUserFunc->iFileToOpen != NULL)
+    {
+        LISPASSERT(multiUserFunc->iFileToOpen->iIsLoaded);
+        }
+        */
+        
     // add an operator with this arity to the multiuserfunc.
     BranchingUserFunction *newFunc;
     if (aListed)
@@ -354,6 +361,27 @@ void LispEnvironment::DeclareRuleBase(LispStringPtr aOperator,
 #endif
 }
 
+void LispEnvironment::DeclareMacroRuleBase(LispStringPtr aOperator, LispPtr& aParameters, LispInt aListed)
+{
+    LispMultiUserFunction* multiUserFunc = MultiUserFunction(aOperator);
+    MacroUserFunction *newFunc;
+    if (aListed)
+    {
+      newFunc = NEW ListedMacroUserFunction(aParameters);
+    }
+    else
+    {
+      newFunc = NEW MacroUserFunction(aParameters);
+    }
+    multiUserFunc->DefineRuleBase(newFunc);
+
+#ifdef YACAS_DEBUG
+    {
+        extern long theNrDefinedUser;
+        theNrDefinedUser++;
+    }
+#endif
+}
 
 
 
@@ -434,20 +462,19 @@ void LispEnvironment::DefineRulePattern(LispStringPtr aOperator,LispInt aArity,
 
 
 
-void LispEnvironment::SetCommand(LispEvalCaller aEvaluatorFunc, LispCharPtr aString)
+void LispEnvironment::SetCommand(YacasEvalCaller aEvaluatorFunc, LispCharPtr aString,LispInt aNrArgs,LispInt aFlags)
 {
 #ifdef YACAS_DEBUG
     extern long theNrDefinedBuiltIn;
     theNrDefinedBuiltIn++;
 #endif
-    LispEvaluator eval(aEvaluatorFunc);
-    Commands().SetAssociation(eval,
-                            HashTable().LookUp(aString,LispTrue));
+    YacasEvaluator eval(aEvaluatorFunc,aNrArgs,aFlags);
+    CoreCommands().SetAssociation(eval,HashTable().LookUp(aString,LispTrue));
 }
 
-void LispEnvironment::RemoveCommand(LispCharPtr aString)
+void LispEnvironment::RemoveCoreCommand(LispCharPtr aString)
 {
-    Commands().Release(HashTable().LookUp(aString,LispTrue));
+    CoreCommands().Release(HashTable().LookUp(aString,LispTrue));
 }
 
 void LispEnvironment::SetUserError(LispCharPtr aErrorString)
@@ -522,6 +549,7 @@ LispCharPtr LispEnvironment::ErrorString(LispInt aError)
         break;
     case KLispErrNonBooleanPredicateInPattern:
         return "Predicate doesn't evaluate to a boolean in pattern";
+     case KLispErrGenericFormat: return GenericErrorBuf();
     }
     return "Unspecified Error";
 }
@@ -545,13 +573,6 @@ LispStringPtr LispEnvironment::FindCachedFile(LispCharPtr aFileName)
 //printf("File %s unsuccessfully decompressed\n",aFileName);
         }
     }
-
-    LispStringPtr hashedname = HashTable().LookUp(aFileName);
-    LispRamFile* ramFile=iRamDisk.LookUp(hashedname);
-    if (ramFile != NULL)
-    {
-        return NEW LispString(ramFile->Contents()->String(),LispTrue);
-    }
     return NULL;
 }
 
