@@ -13,12 +13,13 @@
 class StringInput : public LispInput
 {
 public:
-    StringInput(const LispString aString,InputStatus& aStatus);
+    StringInput(LispString& aString,InputStatus& aStatus);
     virtual LispChar Next();
     virtual LispChar Peek();
     virtual LispBoolean EndOfStream();
     virtual LispCharPtr StartPtr();
     virtual LispInt Position();
+    virtual void SetPosition(LispInt aPosition);
 protected:
     LispString iString;
     LispInt iCurrent;
