@@ -12,7 +12,7 @@ void CUnixCommandLine::NewLine()
 
 void CUnixCommandLine::Pause()
 {
-    int i=clock()+CLOCKS_PER_SEC/4;
+    clock_t i=clock()+CLOCKS_PER_SEC/4;
     while (clock()<i);
 }
 
@@ -53,7 +53,11 @@ CUnixCommandLine::CUnixCommandLine()
     tcsetattr(0, TCSADRAIN, &rl_termio);
 
     char fname[256];
+#ifdef HAVE_VSNPRINTF
+    snprintf(fname,256,"%s/.yacas_history",getenv("HOME"));
+#else
     sprintf(fname,"%s/.yacas_history",getenv("HOME"));
+#endif
     FILE*f=fopen(fname,"r");
     if (f)
     {
@@ -75,14 +79,18 @@ CUnixCommandLine::CUnixCommandLine()
         }
     }
     history=iHistory.NrItems();
-    iMaxLines = 50;
+    iMaxLines = 1024;
 
 }
 CUnixCommandLine::~CUnixCommandLine()
 {
     tcsetattr(0, TCSADRAIN, &orig_termio);
     char fname[256];
+#ifdef HAVE_VSNPRINTF
+    snprintf(fname,256,"%s/.yacas_history",getenv("HOME"));
+#else
     sprintf(fname,"%s/.yacas_history",getenv("HOME"));
+#endif
     
     FILE*f=fopen(fname,"w");
     if (f)
