@@ -11,8 +11,12 @@
 #include <termios.h>
 #include <time.h>
 
+#include "yacasbase.h"
 #include "commandline.h"
 
+/** Unix command line class, using assorted termios functionality
+ *  and sending ansi character sequences to the console.
+ */
 class CUnixCommandLine : public CCommandLine
 {
 public:
@@ -23,9 +27,12 @@ public:
     virtual void NewLine();
     virtual void ShowLine(LispCharPtr prompt,LispInt promptlen,LispInt cursor);
     virtual void Pause();
+    virtual void MaxHistoryLinesSaved(LispInt aNrLines);
 private:
     unsigned char term_chars[NCCS];
     struct termios orig_termio, rl_termio;
+public:
+    LispInt iMaxLines;
 };
 
 
